# kvm

Working on scripting to create a VM from a template.

[Ubuntu 22.04.5 LTS (Jammy Jellyfish)](https://releases.ubuntu.com/jammy/ubuntu-22.04.5-live-server-amd64.iso)

Note that the following steps are derived from this [guide](https://www.pugetsystems.com/labs/hpc/ubuntu-22-04-server-autoinstall-iso/)

[Autoinstall configuration reference manual](https://canonical-subiquity.readthedocs-hosted.com/en/latest/reference/autoinstall-reference.html)

## Generating the autoinstall ISO

1. Install necessary packages

```bash
apt install xorriso
apt install p7zip
```

2. Download the Ubuntu image

```bash
mkdir iso-build
cd iso-build
wget https://releases.ubuntu.com/jammy/ubuntu-22.04.5-live-server-amd64.iso
```

3. Unpack the files and partition the images

```bash
mkdir source-files
7z -y x ubuntu-22.04.5-live-server-amd64.iso -osource-files
```

4. In the source-files directory, you will see the ISO files plus a directory named ‘[BOOT]’, which contains 1-Boot-NoEmul.img and 2-Boot-NoEmul.img. Those are, respectively, the mbr (master boot record) and efi (UEFI) partition images from the ISO. They will be used to create the modified ISO. There is no reason to leave the raw image files on the new ISO, so move them out of the way and give the directory a better name.

```bash
cd source-files
ls
mv  '[BOOT]' ../BOOT
```

5. Edit the grub.cfg file at `source-files/boot/grub/grub.cfg`

```bash
vim boot/grub/grub.cfg
```

Add the following above the existing menu entries:

```bash
menuentry "Autoinstall Ubuntu Server" {
    set gfxpayload=keep
    linux   /casper/vmlinuz quiet autoinstall ds=nocloud\;s=/cdrom/server/  ---
    initrd  /casper/initrd
}
```

This will enable autoinstall and reference the `server` directory where our `user-data` and `meta-data` files will be located.

6. Create and add your custom autoinstall data files

```bash
mkdir server
cd server
touch meta-data
vim user-data
```

Paste the `user-data.yaml` content into the `user-data` file.

7. Generate a new Ubuntu 22.04 server autoinstall ISO

The following command is helpful when trying to set up the arguments for building an ISO. It will give flags and data to closely reproduce the source base install ISO.

```bash
cd ../..
xorriso -indev ubuntu-22.04.5-live-server-amd64.iso -report_el_torito as_mkisofs
```

Using the output of the above, we create the following command. Make sure to run this from the `source-files` directory.

```bash
cd source-files
xorriso -as mkisofs -r \
  -V 'Ubuntu-Server 22.04.5 LTS amd64' \
  -o ../ubuntu-22.04-autoinstall.iso \
  --grub2-mbr ../BOOT/1-Boot-NoEmul.img \
  -partition_offset 16 \
  --mbr-force-bootable \
  -append_partition 2 28732ac11ff8d211ba4b00a0c93ec93b ../BOOT/2-Boot-NoEmul.img \
  -appended_part_as_gpt \
  -iso_mbr_part_type a2a0d0ebe5b9334487c068b6b72699c7 \
  -c '/boot.catalog' \
  -b '/boot/grub/i386-pc/eltorito.img' \
    -no-emul-boot -boot-load-size 4 -boot-info-table --grub2-boot-info \
  -eltorito-alt-boot \
  -e '--interval:appended_partition_2_start_1040737s_size_10072d:all::' \
  -no-emul-boot \
  .
```

If you want to verify the structure of the `user-data` file before running this or to troubleshoot later, the following can be helpful.

```bash
cloud-init schema --config-file server/user-data
```

Alternatively, you can also verify with:

```bash
sudo apt install yamllint
yamllint server/user-data
```

Once the `xorriso` command is run successfully, the `ubuntu-22.04-autoinstall.iso` will be created in the `iso-build` directory.

## Creating the VM using Bolt

Update the parameters provided to the below plan run command as needed.

```bash
cd bolt_vm_automation
bolt plan run bolt_vm_automation::create_ubuntu \
  target=vortex \
  vm_name=moeny-bank01 \
  ip_with_cidr=100.40.223.189/24 \
  hostname=moeny-bank01
```

## Alpine VMs

There are now separate plans for generating a VM using Alpine and Ubuntu. [create_alpine](bolt_vm_automation/plans/create_alpine.yaml) should be run for Alpine and [create_ubuntu](bolt_vm_automation/plans/create_ubuntu.yaml) should be run for Ubuntu. These plans each run tasks tailored for the appropriate distribution.

Below is a sample command to run the Alpine bolt plan.

```bash
bolt plan run bolt_vm_automation::create_alpine \
  vm_name=moeny-service \
  ip_with_cidr=100.40.223.189/24 \
  hostname=moeny-service \
  add_a_record_bool=true \
  dns_hostname=service
```

Note that `add_a_record_bool` must be set to `true` if you would like an A record for the VM to be added to the DNS server zone file, as it is `false` by default. If using this functionality, `dns_hostname` should also be provided and optionally `dns_ttl` if you do not want the default of `3600`. The ability to interact with the DNS server depends on having set up a TSIG key on your DNS server for dynamic updates and storing a copy of your `tsig.key` file in a directory called `keys` at the root of this project. If either of these conditions have not been met, do not attempt to use this functionality. For more information on setting up dynamic DNS with a TSIG key, see our [bind9](https://gitea.moeny.ai/moeny/bind9) repo.

Similarly, `install_docker_bool` can be set to `false` if you do not want docker to be installed on the VM. It is true by default.

For more detailed logging on the `bolt plan run` add the `-v` flag at the end of the command.

If you want to delete an A record that you have added, you can use the [`delete_dns_a_record`](bolt_vm_automation/tasks/delete_dns_a_record.sh) task. You'll just need to provide it with the dns_hostname you set. Here's a sample command.

```bash
bolt task run bolt_vm_automation::delete_dns_a_record dns_hostname=service --targets localhost
```

Lastly, even though it is designed to be run with the `create_alpine` plan, you can also run the [`add_dns_a_record`](bolt_vm_automation/tasks/add_dns_a_record.sh) task on its own. You'll just need to provide it a few parameters. Here's a sample command.

```bash
bolt task run bolt_vm_automation::add_dns_a_record add_a_record_bool=true ip_with_cidr=100.40.223.189/24 dns_hostname=service dns_ttl=3600 --targets localhost
```

Alternatively, to update DNS with the `nsupdate` command directly from the terminal, run something like the following with the path to your `tsig.key`:

```bash
nsupdate -k ../../keys/tsig.key << EOF
server ns1.moeny.ai
zone moeny.ai
update add service.moeny.ai 3600 A 6.5.2.5
send
EOF

ssh moeny@ns1.moeny.ai "sudo rndc sync moeny.ai"
```